#!/usr/bin/env python
"""
This simple example is used for the line-by-line tutorial
that comes with pygame. It is based on a 'popular' web banner.
Note there are comments here, but for the full explanation, 
follow along in the tutorial.
"""


#Import Modules
import os, pygame, operator
from copy import copy
from copy import deepcopy
from pygame.locals import *

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

def allKUPs(x): return x.type == KEYUP

#-----------------------------------------------------------------------------
def load_charactor_defs(name):
	exec( 'from data.'+name+'.chardef import charAnims' )
	return charAnims

	
#-----------------------------------------------------------------------------
def load_png(name, extradir=''):
    fullname = os.path.join('data', extradir, name)
    try:
        image = pygame.image.load(fullname)
	if image.get_alpha is None:
		image = image.convert()
	else:
		image = image.convert_alpha()
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    return image, image.get_rect()

#-----------------------------------------------------------------------------
def load_sound(name):
    class NoneSound:
        def play(self): pass
    if not pygame.mixer or not pygame.mixer.get_init():
        return NoneSound()
    fullname = os.path.join('data', name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except pygame.error, message:
        print 'Cannot load sound:', fullname
        raise SystemExit, message
    return sound

#-----------------------------------------------------------------------------
class SimpleSprite(pygame.sprite.Sprite):
	def __init__(self, imgName=None):
		pygame.sprite.Sprite.__init__(self) #call Sprite initializer
		if imgName:
			self.image, self.rect = load_png(imgName)
		else:
			self.image = pygame.Surface( (100,100) )
			self.image.fill( (200,0,0) )
			self.rect = self.image.get_rect()
	def update(self):
		pass
	def SetImg(self, newSurface, newRect=None):
		self.image = newSurface
		if newRect:
			self.rect  = newRect
		else:
			self.rect  = newSurface.get_rect()


#-----------------------------------------------------------------------------
class StatusBar(pygame.sprite.Sprite):
	"""status bar to track health"""
	def __init__(self, charactor, rect, bottomGroup, topGroup,
	             outlineImg=None, insideImg=None):
		"""pass in a rectstyle to tell the statusbar where to appear
		   and pass in an object with a health attribute"""
		pygame.sprite.Sprite.__init__(self) #call Sprite initializer

		if outlineImg is not None:
			self.image, self.rect = load_png(outlineImg)
			self.rect.move_ip( rect.topleft )
		else:
			self.rect = pygame.Rect(rect)
			self.image = pygame.Surface( (self.rect.width,
			                              self.rect.height) )
			self.image.fill( (0,0,0) )
		self.add( bottomGroup )

		if insideImg is not None:
			self.amtSprite = SimpleSprite(insideImg)
		else:
			self.amtSprite = SimpleSprite()
			self.amtSprite.rect = self.rect.inflate( -10, -10 )
			self.amtSprite.image.fill( (220,0,0) )
		self.amtSprite.add( topGroup )

		self.charactor = charactor
		self.fullAmt = charactor.health
		self.charactor.SetStatusBar( self )
		#self.rect.move_ip( 50, 10 )
		self.amt = 100
		self.UpdateImg()

	def update(self):
		pass

	def UpdateImg(self):
		#print "full: ", self.fullAmt, " new: ", self.charactor.health
		amt = self.charactor.health / float(self.fullAmt)
		height = self.amtSprite.rect.height
		fullWidth  = self.amtSprite.rect.width

		width = fullWidth * amt
		if width < 0:
			width = 0

		if self.amtSprite.image.get_rect().width <= 0:
			groups = self.amtSprite.groups()
			self.amtSprite.kill()
			self.amtSprite = SimpleSprite()
			self.amtSprite.rect = self.rect.inflate( -10, -10 )
			self.amtSprite.image.fill( (220,0,0) )
			self.amtSprite.add( groups )

		print "img W ", self.amtSprite.image.get_rect().width 
		print "w", width, " h", height
		self.amtSprite.image = \
		                  pygame.transform.scale( self.amtSprite.image,
				                          (width,height) )

		#groupList = self.groups()
		#self.amtSprite.add( groupList )
		#print self.amtSprite

#classes for our game objects
#-----------------------------------------------------------------------------
class PunchGame:
	def __init__(self, charGroup, hudGroup ):
		self.charGroup = charGroup
		self.hudGroup = hudGroup

		self.p1 = Charactor( self, 'c1', [200,200])
		self.p2 = Charactor( self, 'c2', [600,200])

		self.p1.SetEnemy( self.p2 )
		self.p2.SetEnemy( self.p1 )

		listenDict = { K_w: self.p1.MoveUp,
				    K_s: self.p1.MoveDown,
				    K_a: self.p1.MoveLeft,
				    K_d: self.p1.MoveRight,
				    K_SPACE: self.p1.Punch
				  }

		offDict = { K_w: self.p1.StopUp,
				    K_s: self.p1.StopDown,
				    K_a: self.p1.StopLeft,
				    K_d: self.p1.StopRight,
				    K_SPACE: self.p1.UnPunch
				  }
		self.p1.SetKeys( listenDict, offDict )

		p1Status = StatusBar( self.p1, (100,500,200,50),
		                      self.charGroup, self.hudGroup )

		p2Status = StatusBar( self.p2, (500,500,200,50),
		                      self.charGroup, self.hudGroup )

		self.charGroup.add( self.p1.getAllSprites() +
		                     self.p2.getAllSprites() )

		self.difficulty = 1.0
		self.listenKeys = {}

		self.active = 1

		self.allowSignalsThrough = 1

	def Restart(self):
		self.active = 1
		self.allowSignalsThrough = 1
		self.p1.Restart()
		self.p2.Restart()

		self.buttonsprite.kill()
		Garbage.add( self.buttonsprite )
		self.textsprite.kill()
		Garbage.add( self.textsprite )


	def OnCharactorDeath(self, deadChar):
		self.difficulty = self.difficulty + 0.25

		self.allowSignalsThrough = 0

		self.textsprite = pygame.sprite.Sprite( self.hudGroup )
		font = pygame.font.Font(None, 60)
		self.textsprite.image = font.render("GAME OVER!", 1, (10,10,10))
		self.textsprite.rect = self.textsprite.image.get_rect()
		self.textsprite.rect.centerx = screen.get_rect().centerx

		self.buttonsprite = SimpleSprite('playagain.png')
		self.buttonsprite.rect.centerx = screen.get_rect().centerx
		self.buttonsprite.rect.centery = screen.get_rect().centery
		self.hudGroup.add( self.buttonsprite )

		self.active = 0


	def SignalKey( self, event, remainingEvents ):
		if self.listenKeys.has_key( event.key ):
			self.listenKeys[event.key]()
		elif self.allowSignalsThrough:
			self.p1.SignalKey( event, remainingEvents )
			self.p2.SignalKey( event, remainingEvents )
		
		
#-----------------------------------------------------------------------------
class BoundingBox:
	def __init__(self):
		self.top = None;
		self.right = None;
		self.bottom = None;
		self.left = None;

		self.topSprite = None;
		self.rightSprite = None;
		self.bottomSprite = None;
		self.leftSprite = None;

#-----------------------------------------------------------------------------
class Charactor(pygame.sprite.Sprite):
    """moves a charactor on the screen, following the mouse"""
    def __init__(self, game, name, startpos=[600,200]):
        pygame.sprite.Sprite.__init__(self) #call Sprite initializer
	self.game = game
	self.name = name
	self.anims = load_charactor_defs(name)
	self.anim = self.anims.ChangeTo('rest')
	self.frame = self.anim.first()
        self.rect = self.frame.rect
        self.image = self.frame.image
	#self.rect = load_png('c1/rest01.png')
	#self.rightFacingImg = self.image
	#self.leftFacingImg = pygame.transform.flip( self.image, 1, 0)

	self.health = 100

	self.rect.midbottom = startpos
	self.startpos = startpos
	self.rect.inflate_ip(-2,-2)
        self.area = screen.get_rect().inflate(50,50)
	self.oldPos = self.rect

	self.listenKeys = { K_UP: self.MoveUp,
	                    K_DOWN: self.MoveDown,
	                    K_LEFT: self.MoveLeft,
	                    K_RIGHT: self.MoveRight,
	                    K_RETURN: self.Punch
	                  }

	self.offKeys = { K_UP: self.StopUp,
	                    K_DOWN: self.StopDown,
	                    K_LEFT: self.StopLeft,
	                    K_RIGHT: self.StopRight,
	                    K_RETURN: self.UnPunch
	                  }

	self.offset = [0,0]
	self.facing = 0
	self.moveState = [0,0]
	self.speed     = [10,10]
	self.accelerate = 0
	self.jumping = 0
	self.ducking = 0
	self.falling = 1
	self.snd_jump = load_sound('jump.wav')
	self.snd_land = load_sound('land.wav')
	self.snd_htop = load_sound('top.wav')
	self.snd_swing = load_sound('land.wav')
	self.snd_hit = load_sound('whiff.wav')
    
    #--------------------------------------------------------------------------
    def Restart(self):
	self.rect.midbottom = self.startpos
	self.offset = [0,0]
	self.facing = 0
	self.moveState = [0,0]
	self.speed     = [10,10]
	self.accelerate = 0
	self.jumping = 0
	self.ducking = 0
	self.falling = 1

	self.health = 100
	if self.statusBar:
		self.statusBar.UpdateImg()

    #--------------------------------------------------------------------------
    def update(self):
        "move the charactor based on the mouse position"

	enemyRect = self.enemy.rect
	if self.facing is 0 and enemyRect.right < self.rect.left :
		#print self.name, "was facing right, turning left"
		self.facing = 1
		self.anim.SetFacing( self.facing )
	if self.facing is 1 and enemyRect.left > self.rect.right :
		#print self.name, "was facing left, turning right"
		self.facing = 0
		self.anim.SetFacing( self.facing )

	if self.anim.length() > 1:
		self.anim.SetMasterRect( self.rect )
		self.frame = self.anim.next()
		#frame.rect = self.rect
		change = self.frame.GetOffset( self.offset )
		if change != [0,0]:
			#print 'moving by ', change
			self.rect.move_ip( change )
			self.offset = map(operator.add, self.offset, change )
	
		
	self.Move()


	#frame.rect.move_ip( self.rect.left, self.rect.top )
	#print "f", frame.rect

	#hitbox = frame.attRect
	#hit = hitbox.colliderect(enemyRect)
	if self.frame.Hit( self.enemy ):
		self.Attack();


	self.AccelStep()
	if self.jumping:
		self.JumpStep()
	if self.falling:
		self.FallStep()

	self.image = self.frame.image

    #--------------------------------------------------------------------------
    def SetKeys(self, listenDict, offDict):
	self.listenKeys = listenDict
	self.offKeys = offDict

    #--------------------------------------------------------------------------
    def SetEnemy(self, enemy):
	self.enemy = enemy

    #--------------------------------------------------------------------------
    def SetStatusBar(self, statBar):
    	self.statusBar = statBar

    #--------------------------------------------------------------------------
    def GetFrame(self):
	return self.frame

    #--------------------------------------------------------------------------
    def Move( self ):
	#the change in position is the direction they're moving times the
	# speed they're going
    	change = map(operator.mul, self.moveState, self.speed )

	if change == [0,0]:
		return

	#print change

        self.oldPos = self.rect.move( (0,0) )
        self.rect.move_ip( change )
        newPos = self.rect

	#the position is limited by the maximum allowed area
	global Solids
	limiter = self.Blocked( newPos, Solids )
	if limiter.top is not None:
		self.rect.top = limiter.top
		self.BumpTop( limiter.topSprite )
	if limiter.right is not None:
		self.rect.right = limiter.right
	if limiter.bottom is not None:
		self.rect.bottom = limiter.bottom
		self.BumpBottom( limiter.bottomSprite )
	if limiter.left is not None:
		self.rect.left = limiter.left
	

	
    #--------------------------------------------------------------------------
    def Blocked( self, newPos, spriteGroup ):
	limiter = BoundingBox()
	oldPos = self.oldPos

	if newPos.top    <= self.area.top:
		limiter.top = self.area.top
	if newPos.right  >= self.area.right:
		limiter.right = self.area.right
	if newPos.bottom >= self.area.bottom:
		limiter.bottom = self.area.bottom
	if newPos.left   <= self.area.left:
		limiter.left = self.area.left

	colliders = pygame.sprite.spritecollide( self, spriteGroup, 0)


	if len(colliders)  is not 0:
		for sprite in colliders:

			if ( limiter.top is None \
			   or sprite.rect.bottom > limiter.top ) \
			   and newPos.top      < sprite.rect.bottom \
			   and oldPos.top        >= sprite.rect.bottom:
				limiter.top        = sprite.rect.bottom
				limiter.topSprite  = sprite

			if ( limiter.right is None  \
			   or sprite.rect.left  < limiter.right ) \
			   and newPos.right    > sprite.rect.left \
			   and oldPos.right      <= sprite.rect.left:
				limiter.right        = sprite.rect.left
				limiter.rightSprite  = sprite

			if ( limiter.bottom is None \
			   or sprite.rect.top < limiter.bottom ) \
			   and newPos.bottom > sprite.rect.top \
			   and oldPos.bottom   <= sprite.rect.top:
				limiter.bottom       = sprite.rect.top
				limiter.bottomSprite = sprite

			if ( limiter.left is None \
			   or sprite.rect.right  > limiter.left ) \
			   and newPos.left   < sprite.rect.right \
			   and oldPos.left     >= sprite.rect.right:
				limiter.left       = sprite.rect.right
				limiter.leftSprite = sprite

	return limiter


    #--------------------------------------------------------------------------
    def MoveLeft( self, remainingEvents ):
    	self.accelerate = -1
    	#if self.moveState[0] > -1:
		#self.moveState[0] = self.moveState[0] - 0.5 

    #--------------------------------------------------------------------------
    def MoveRight( self, remainingEvents ):
    	self.accelerate = 1
	#self.moveState[0] = 1

    #--------------------------------------------------------------------------
    def MoveUp( self, remainingEvents ):
	if not self.jumping:
		self.Jump()

    #--------------------------------------------------------------------------
    def Jump( self ):
	self.snd_jump.play()
	self.jumping = 1
	self.falling = 0
	self.moveState[1] = self.moveState[1] -2

    #--------------------------------------------------------------------------
    def AccelStep( self ):
    	# 1 = right, -1 = left
    	if self.accelerate != 0 and abs(self.moveState[0]) >=2:
		return
	if self.accelerate == 0:
		self.moveState[0] = self.moveState[0] * 0.8
	else:
		self.moveState[0] = self.moveState[0] + ( 0.2*self.accelerate )

    #--------------------------------------------------------------------------
    def JumpStep( self ):
    	if self.moveState[1] > 0:
		self.jumping = 0
		self.falling = 1
		return
	self.moveState[1] = self.moveState[1] + 0.5

    #--------------------------------------------------------------------------
    def FallStep( self ):
	if self.moveState[1] < 4:
		self.moveState[1] = self.moveState[1] + 0.2

    	change = map(operator.mul, self.moveState, self.speed )
	if change == [0,0]:
		return

        newPos = self.rect.move( change )
	enemyGroup = pygame.sprite.Group(self.enemy)
	lim = self.Blocked( newPos, enemyGroup )
	enemyGroup.empty()
	if lim.bottomSprite is not None:
		self.moveState[1] = -1

    #--------------------------------------------------------------------------
    def BumpBottom(self, bottomSprite ):
    	if self.moveState[1] > 2:
		self.snd_land.play()
	#self.falling = 0
	self.moveState[1] = 0

	#did we just fall off the screen?
	if self.rect.bottom >= self.area.bottom:
		self.Die()

    #--------------------------------------------------------------------------
    def BumpTop(self, topSprite ):
	self.snd_htop.play()
	self.jumping = 0
	self.falling = 1
	self.moveState[1] = 0



    #--------------------------------------------------------------------------
    def MoveDown( self, remainingEvents ):
	if not self.jumping:
		self.ducking = 1

    #--------------------------------------------------------------------------
    def StopLeft( self, remainingEvents ):
    	self.accelerate = 0 
	#if self.moveState[0] is -1:
		#self.moveState[0] = 0
    
    #--------------------------------------------------------------------------
    def StopRight( self, remainingEvents ):
    	self.accelerate = 0 
    
    #--------------------------------------------------------------------------
    def StopUp( self, remainingEvents ):
	return
	if self.moveState[1] is -1:
		self.moveState[1] = 0
    
    #--------------------------------------------------------------------------
    def StopDown( self, remainingEvents ):
	self.ducking = 0
	if self.moveState[1] is 1:
		self.moveState[1] = 0
    
    #--------------------------------------------------------------------------
    def Punch(self, remainingEvents):
	self.snd_swing.play()
    	self.anim = self.anims.ChangeTo('punch')
	self.anim.SetFacing( self.facing )
	return

    #--------------------------------------------------------------------------
    def Attack(self):
    	self.enemy.Punched(damage=10)

    #--------------------------------------------------------------------------
    def UnPunch(self, remainingEvents ):
        "called to pull the charactor back"
    	self.anim = self.anims.ChangeTo('rest')
	self.anim.SetFacing( self.facing )
	return

    #--------------------------------------------------------------------------
    def Punched(self, damage=30):
	self.snd_hit.play()
	self.health = self.health - damage
	if self.health < 0:
		self.health = 0
		self.Die()
	if self.statusBar:
		self.statusBar.UpdateImg()

    #--------------------------------------------------------------------------
    def Die(self):
	self.health = 0
	self.falling = 0
	self.jumping = 0
	self.accelerate = 0
	self.speed = [0,0]
	self.moveState = [0,0]
	self.game.OnCharactorDeath( self )
	if self.statusBar:
		self.statusBar.UpdateImg()

    #--------------------------------------------------------------------------
    def getAllSprites(self):
	return ( self,)

    #--------------------------------------------------------------------------
    def SignalKey( self, event, remainingEvents ):
		if self.listenKeys.has_key( event.key ) \
		  and event.type is KEYDOWN:
			self.listenKeys[event.key]( remainingEvents )
		elif self.offKeys.has_key( event.key ) \
		  and event.type is KEYUP:
			self.offKeys[event.key]( remainingEvents  )








#-----------------------------------------------------------------------------
class GarbageSpriteGroup(pygame.sprite.RenderClear):

	def GetAllRects(self):
		rectlist = []
		for sprite in self.sprites():
			rectlist.append( sprite.rect )
		return rectlist

#-----------------------------------------------------------------------------
def main():
	"""this function is called when the program starts.
	   it initializes everything it needs, then runs in
	   a loop until the function returns."""
#Initialize Everything
	pygame.init()
	global screen
	screen = pygame.display.set_mode((800, 600))
	pygame.display.set_caption('Sucka Jab Bots')
	#pygame.mouse.set_visible(0)

#Create The Backgound
	global bgsurface
	bgsurface, garbage = load_png('backgr.png')

#Display The Background
	screen.blit(bgsurface, (0, 0))
	pygame.display.flip()


#Prepare Game Objects
	clock = pygame.time.Clock()

	global Solids
	Solids = pygame.sprite.RenderUpdates()
	global Garbage
	Garbage = GarbageSpriteGroup()
	allsprites = pygame.sprite.RenderUpdates()
	hudSprites = pygame.sprite.RenderUpdates()
	Solids = pygame.sprite.RenderUpdates()

#put in the two platforms to walk on
	plat1 = SimpleSprite('plat1.png')
	plat1.rect.move_ip( 73, 286 )
	Solids.add( plat1 )

	keysprite = SimpleSprite('keybd.png')
	keysprite.rect.centerx = screen.get_rect().centerx
	hudSprites.add( keysprite )
	textsprite = pygame.sprite.Sprite( hudSprites )
	font = pygame.font.Font(None, 60)
	textsprite.image = font.render(str(0), 1, (10,10,10))
	textsprite.rect = textsprite.image.get_rect()
	textsprite.rect.centerx = screen.get_rect().centerx

	i = 3
	while i > 0:
		textsprite.image = font.render(str(i), 1, (10,10,10))
		hudSprites.clear( screen, bgsurface )
		changedRects1 = hudSprites.draw(screen)
		pygame.display.update( changedRects1 )
		clock.tick(1)
		i = i -1

	keysprite.kill()
	textsprite.kill()
	Garbage.add( keysprite )
	Garbage.add( textsprite )


	game  = PunchGame(allsprites, hudSprites)

#Main Loop
	oldfps = 0
	while 1:
		clock.tick(40)
		newfps = int(clock.get_fps())
		if newfps is not oldfps:
			print "fps: ", clock.get_fps()
			oldfps = newfps

	#Handle Input Events
		oldEvents = []
		remainingEvents = pygame.event.get()
		for event in remainingEvents:
			oldEvents.append( remainingEvents.pop(0) )
			upKeys = filter( allKUPs, remainingEvents )
			#print event
			if event.type == QUIT:
				return
			elif event.type == KEYDOWN and event.key == K_ESCAPE:
				return
			elif event.type == MOUSEBUTTONDOWN and not game.active:
				game.Restart()
			elif event.type == KEYDOWN or event.type == KEYUP:
				if len(upKeys) > 0:
					print upKeys 
				game.SignalKey( event, upKeys )

		allsprites.clear( screen, bgsurface )
		hudSprites.clear( screen, bgsurface )
		Solids.clear( screen, bgsurface )

		Garbage.clear( screen, bgsurface )
		changedRects4 = Garbage.GetAllRects()
		Garbage.empty()

		allsprites.update()
		hudSprites.update()

	#Draw Everything

		changedRects3 = Solids.draw(screen)
		changedRects2 = allsprites.draw(screen)
		changedRects1 = hudSprites.draw(screen)
		pygame.display.update( changedRects1 \
		                       + changedRects2 \
		                       + changedRects3 \
		                       + changedRects4 )

#Game Over


#this calls the 'main' function when this script is executed
if __name__ == '__main__': main()
